@@ -37,14 +37,14 @@ buffer_grow (bufp, size)
     {
       if (size < 120)
 	size = 120;
-      bufp->data = (unsigned char*) xmalloc (size);
+      bufp->data = xmalloc (size);
       bufp->ptr = bufp->data;
     }
   else
     {
       int index = bufp->ptr - bufp->data;
       size += 2 * (bufp->limit - bufp->data);
-      bufp->data = (unsigned char *) xrealloc (bufp->data, size);
+      bufp->data = xrealloc (bufp->data, size);
       bufp->ptr = bufp->data + index;
     }
   bufp->limit = bufp->data + size;
